/*
	ButtonPlugin.cpp
	Copyright 1997 Nathan Dwyer, All Rights Reserved.
*/


#include "PluginAPI.h"
#include "ViewEditorBase.h"


const EDItemType EType_Button = 'DiBu';
const EDItemType EType_Checkbox = 'DiCh';
const EDItemType EType_RadioButton = 'DiRb';
	
const ulong EType_NumTypes = 3;



static SItemInfo ControlInfo[ EType_NumTypes ] =
{
	{ EType_Button, "button", { 'B', '\0' }, 15, 25, 0, 25, -2, -2, 2, 0, 60, 25, FALSE, 192, { 0, 0 }, 0 },
	{ EType_Checkbox, "checkbox", { 'C', '\0' }, 13, 16, 0, 16, -1, 0, 1, 0, 100, 16, FALSE, 168, { 0, 0 }, 0 },
	{ EType_RadioButton, "radiobutton", { 'R', '\0' }, 14, 18, 0, 18, -1, -1, 1, 0, 100, 18, FALSE, 168, { 0, 0 }, 0 },
};



class CEditor : public CEditorWindow
{
	public:
		CEditor( CDragFrame *pFrame, const char *rsrcData, ulong rsrcLen );

		virtual void Init( void );
		
	protected:
		virtual void _Apply( void );
};



#pragma export on


SItemInfo *GetViewInfo( long *pCnt )
{
	*pCnt = EType_NumTypes;
	return ControlInfo;
}




BView *MakeView( long type, BRect frame )
{
	BView *pControl = NULL;
	
	switch( type )
	{
		case EType_Button:
			pControl = new BButton( frame, NULL, "Button", NULL );
			break;
			
		case EType_Checkbox:
			pControl = new BCheckBox( frame, NULL, "Check Box", NULL );
			break;
			
		case EType_RadioButton:
			pControl = new BRadioButton( frame, NULL, "Radio Button", NULL );
			break;
	}
	
	return pControl;

}


void ReadView( CDragFrame *pFrame, DBlock& buffer, long offset )
{
	long l;
	BControl *pControl = (BControl *) pFrame->GetControl();

	// Read label
	char name[ kDITLControlTitleLen ];
	strncpy( name, buffer( offset + kDITLControlTitle, kDITLControlTitleLen ).Data( l ), sizeof(name) );
	pControl->SetLabel( name );

	// FIX Read message tag
		
	// read IsDefault flag for button
	if( pFrame->GetTypeInfo()->type == EType_Button )
	{
		long isDefault = *(long *) buffer( offset + kDITLButtonDefault, sizeof(long) ).Data( l );;
		((BButton *) pControl)->MakeDefault( isDefault );
	}
}


void WriteView( CDragFrame *pFrame, DBlock& buffer, long offset )
{
	BControl *pControl = (BControl *) pFrame->GetControl();
	
	// write out label for BBox
	char buff[ kDITLControlTitleLen ];
	memset( buff, 0, sizeof( buff ) );
	strcpy( buff, pControl->Label() );
	buffer( offset + kDITLControlTitle, kDITLControlTitleLen ) = DBlock( buff, kDITLControlTitleLen );
	
	// Write message tag
	long msg = 0L;
	buffer( offset + kDITLControlMessage, sizeof(long) ) = DBlock( (char *) &msg, sizeof(long) );
	
	// Write out IsDefault flag for button
	if( pFrame->GetTypeInfo()->type == EType_Button )
	{
		long isDefault = ((BButton *) pControl)->IsDefault();
		buffer( offset + kDITLButtonDefault, sizeof(long) ) = DBlock( (char *) &isDefault, sizeof(long) );
	}
}


BWindow *EditView( CDragFrame *pFrame )
{
	char *rsrcData = NULL;
	long rsrcLen = 0L;
	
	// find record ref for this frame type
	const SItemInfo *pInfo = pFrame->GetTypeInfo();
	
	// get resource
	BResourceFile rsrcFile( pInfo->pluginRef );
	if( B_NO_ERROR == rsrcFile.Open( B_READ_ONLY ) )
	{
		long dlgID = pInfo->type == EType_Button ? 0 : 1;
			
		rsrcData = (char *) rsrcFile.FindResource( 'DITL', dlgID, &rsrcLen );
		rsrcFile.Close();
	}
	
	CEditorWindow *pWin = NULL;
	
	if( rsrcData )
	{
		// create and show editor window
		pWin = new CEditor( pFrame, rsrcData, rsrcLen );	
		pWin->Init();
		
		// clean up
		free( rsrcData );
	}		
	
	return pWin;
}

#pragma export reset


//-------------------------------- Editor Class

CEditor::CEditor( CDragFrame *pFrame, const char *rsrcData, ulong rsrcLen ) :
	CEditorWindow( pFrame, rsrcData, rsrcLen )
{
}
	


void CEditor::Init( void )
{
	Lock();
	
	BView *pControl = GetFrame()->GetControl();
	
	// set name
	BTextControl *pText = (BTextControl *) FindView( "TX_name" );
	if( pText )
	{
		char name[ kDITLViewNameLen ];
		strncpy( name, pControl->Name(), kDITLViewNameLen );
		pText->SetText( name );
	}
		
	// set resize
	ulong resize = pControl->ResizingMode();
	
	const ulong kResizeLRMask = 0x00000f0f;
	if( ( resize & kResizeLRMask ) == B_FOLLOW_LEFT )
		((BRadioButton *) FindView( "RB_left" ))->SetValue( TRUE );
	else if( ( resize & kResizeLRMask ) == B_FOLLOW_RIGHT )
		((BRadioButton *) FindView( "RB_right" ))->SetValue( TRUE );
	else if( ( resize & kResizeLRMask ) == B_FOLLOW_LEFT_RIGHT )
		((BRadioButton *) FindView( "RB_leftright" ))->SetValue( TRUE );
	else
		((BRadioButton *) FindView( "RB_lrcenter" ))->SetValue( TRUE );
		
	const ulong kResizeTBMask = 0x0000f0f0;
	if( ( resize & kResizeTBMask ) == B_FOLLOW_TOP )
		((BRadioButton *) FindView( "RB_top" ))->SetValue( TRUE );
	else if( ( resize & kResizeTBMask ) == B_FOLLOW_BOTTOM )
		((BRadioButton *) FindView( "RB_bottom" ))->SetValue( TRUE );
	else if( ( resize & kResizeTBMask ) == B_FOLLOW_TOP_BOTTOM )
		((BRadioButton *) FindView( "RB_topbottom" ))->SetValue( TRUE );
	else
		((BRadioButton *) FindView( "RB_tbcenter" ))->SetValue( TRUE );

	// Set label
	BTextControl *pLabel = (BTextControl *) FindView( "TX_label" );
	printf( "label: %p\n", pLabel );
	if( pLabel )
	{
		char name[ kDITLControlTitleLen ];
		strncpy( name, ((BControl *) pControl)->Label(), kDITLControlTitleLen );
		pLabel->SetText( name );
	}
	
	if( GetFrame()->GetTypeInfo()->type == EType_Button )
	{
		BCheckBox *pCheck = (BCheckBox *) FindView( "CB_default" );
		if( pCheck )
			pCheck->SetValue( ((BButton *) pControl )->IsDefault() );
	}
	
	Unlock();
}				


void CEditor::_Apply( void )
{
	BView *pControl = GetFrame()->GetControl();
	
	// update name
	BTextControl *pText = (BTextControl *) FindView( "TX_name" );
	if( pText )
	{
		char name[ kDITLViewNameLen ];
		strncpy( name, pText->Text(), kDITLViewNameLen );
		pControl->SetName( name );
	}
		
	// update resize
	BRadioButton *pRad;
	ulong resize = 0L;
	
	if( ((BRadioButton *) FindView( "RB_left" ))->Value() )
		resize = B_FOLLOW_LEFT;
	else if( ((BRadioButton *) FindView( "RB_right" ))->Value() )
		resize = B_FOLLOW_RIGHT;
	else if( ( pRad = (BRadioButton *) FindView( "RB_leftright" ) ) && pRad->Value() )
		resize = B_FOLLOW_LEFT_RIGHT;
	else
		resize = B_FOLLOW_H_CENTER;
	
	if( ((BRadioButton *) FindView( "RB_top" ))->Value() )
		resize |= B_FOLLOW_TOP;
	else if( ((BRadioButton *) FindView( "RB_bottom" ))->Value() )
		resize |= B_FOLLOW_BOTTOM;
	else if( ( pRad = (BRadioButton *) FindView( "RB_topbottom" ) ) && pRad->Value() )
		resize |= B_FOLLOW_TOP_BOTTOM;
	else
		resize |= B_FOLLOW_V_CENTER;

	GetFrame()->SetResizingMode( resize );
	pControl->SetResizingMode( resize );
	
	// Set label
	BTextControl *pLabel = (BTextControl *) FindView( "TX_label" );
	if( pLabel )
	{
		char name[ kDITLControlTitleLen ];
		strncpy( name, pLabel->Text(), kDITLControlTitleLen );
		((BControl *) pControl)->SetLabel( name );
	}
	
	if( GetFrame()->GetTypeInfo()->type == EType_Button )
	{
		BCheckBox *pCheck = (BCheckBox *) FindView( "CB_default" );
		if( pCheck )
			((BButton *) pControl)->MakeDefault( pCheck->Value() );
	}
	
}
