/*
	TextPlugin.cpp
	Copyright 1997 Nathan Dwyer, All Rights Reserved.
*/


#include "PluginAPI.h"
#include "ViewEditorBase.h"


const EDItemType EType_TextControl = 'DiTv';
const EDItemType EType_StringView = 'DiSv';
	
const ulong EType_NumTypes = 2;



static SItemInfo ControlInfo[ EType_NumTypes ] =
{
	{ EType_StringView, "stringview", { 'S', '\0' }, 10, 14, 0, 0, 0, 0, 0, 0, 60, 14, FALSE, 168, { 0, 0 }, 0 },
	{ EType_TextControl, "textcontrol", { 'T', '\0' }, 10, 17, 0, 0, 0, -1, 2, 1, 73, 17, FALSE, 168, { 0, 0 }, 0 },
};



class CEditor : public CEditorWindow
{
	public:
		CEditor( CDragFrame *pFrame, const char *rsrcData, ulong rsrcLen );

		virtual void Init( void );
		
	protected:
		virtual void _Apply( void );
};



#pragma export on


SItemInfo *GetViewInfo( long *pCnt )
{
	*pCnt = EType_NumTypes;
	return ControlInfo;
}




BView *MakeView( long type, BRect frame )
{
	BView *pControl = NULL;
	
	switch( type )
	{
		case EType_TextControl:
			pControl = new BTextControl( frame, NULL, "Label:", "Text", NULL );
			break;
			
		case EType_StringView:
			pControl = new BStringView( frame, NULL, "String View" );
			break;
			
	}
	
	return pControl;

}


void ReadView( CDragFrame *pFrame, DBlock& buffer, long offset )
{
	long l;
	
	if( pFrame->GetTypeInfo()->type == EType_StringView )
	{
		BStringView *pView = (BStringView *) pFrame->GetControl();
	
		// Read label
		char name[ kDITLStringTextLen ];
		strncpy( name, buffer( offset + kDITLStringText, kDITLStringTextLen ).Data( l ), sizeof(name) );
		pView->SetText( name );
		
		// read alignment
		alignment al = *(alignment *) buffer( offset + kDITLStringAlign, sizeof(long) ).Data( l );
		pView->SetAlignment( al );
	}
	else
	{
		BTextControl *pText = (BTextControl *) pFrame->GetControl();
	
		// Read label
		char name[ kDITLTextCLabelLen ];
		strncpy( name, buffer( offset + kDITLTextCLabel, kDITLTextCLabelLen ).Data( l ), sizeof(name) );
		pText->SetLabel( name );
		
		// read divider
		float div = *(float *) buffer( offset + kDITLTextCDivider, sizeof(float) ).Data( l );
		pText->SetDivider( div );
	}
}


void WriteView( CDragFrame *pFrame, DBlock& buffer, long offset )
{
	if( pFrame->GetTypeInfo()->type == EType_StringView )
	{
		BStringView *pView = (BStringView *) pFrame->GetControl();
	
		// Write label
		char name[ kDITLStringTextLen ];
		memset( name, 0, sizeof(name) );
		strcpy( name, pView->Text() );
		buffer( offset + kDITLStringText, kDITLStringTextLen ) = DBlock( name, kDITLStringTextLen );	
		
		// write alignment
		long al = pView->Alignment();
		buffer( offset + kDITLStringAlign, sizeof(long) ) = DBlock( (char *) &al, sizeof(long) );
	}
	else
	{
		BTextControl *pText = (BTextControl *) pFrame->GetControl();
	
		// Write label
		char name[ kDITLTextCLabelLen ];
		memset( name, 0, sizeof(name) );
		strcpy( name, pText->Label() );
		buffer( offset + kDITLTextCLabel, kDITLTextCLabelLen ) = DBlock( name, kDITLTextCLabelLen );	
		
		// Write divider
		float d = pText->Divider();
		buffer( offset + kDITLTextCDivider, sizeof(float) ) = DBlock( (char *) &d, sizeof(float) );	
	}
}


BWindow *EditView( CDragFrame *pFrame )
{
	char *rsrcData = NULL;
	long rsrcLen = 0L;
	
	// find record ref for this frame type
	const SItemInfo *pInfo = pFrame->GetTypeInfo();
	
	// get resource
	BResourceFile rsrcFile( pInfo->pluginRef );
	if( B_NO_ERROR == rsrcFile.Open( B_READ_ONLY ) )
	{
		long dlgID = pInfo->type == EType_StringView ? 0 : 1;
			
		rsrcData = (char *) rsrcFile.FindResource( 'DITL', dlgID, &rsrcLen );
		rsrcFile.Close();
	}
	
	CEditorWindow *pWin = NULL;
	
	if( rsrcData )
	{
		// create and show editor window
		pWin = new CEditor( pFrame, rsrcData, rsrcLen );	
		pWin->Init();
		
		// clean up
		free( rsrcData );
	}		
	
	return pWin;
}

#pragma export reset


//-------------------------------- Editor Class

CEditor::CEditor( CDragFrame *pFrame, const char *rsrcData, ulong rsrcLen ) :
	CEditorWindow( pFrame, rsrcData, rsrcLen )
{
}
	


void CEditor::Init( void )
{
	Lock();
	
	BView *pControl = GetFrame()->GetControl();
	
	// set name
	BTextControl *pText = (BTextControl *) FindView( "TX_name" );
	if( pText )
	{
		char name[ kDITLViewNameLen ];
		strncpy( name, pControl->Name(), kDITLViewNameLen );
		pText->SetText( name );
	}
		
	// set resize
	ulong resize = pControl->ResizingMode();
	
	const ulong kResizeLRMask = 0x00000f0f;
	if( ( resize & kResizeLRMask ) == B_FOLLOW_LEFT )
		((BRadioButton *) FindView( "RB_left" ))->SetValue( TRUE );
	else if( ( resize & kResizeLRMask ) == B_FOLLOW_RIGHT )
		((BRadioButton *) FindView( "RB_right" ))->SetValue( TRUE );
	else if( ( resize & kResizeLRMask ) == B_FOLLOW_LEFT_RIGHT )
		((BRadioButton *) FindView( "RB_leftright" ))->SetValue( TRUE );
	else
		((BRadioButton *) FindView( "RB_lrcenter" ))->SetValue( TRUE );
		
	const ulong kResizeTBMask = 0x0000f0f0;
	if( ( resize & kResizeTBMask ) == B_FOLLOW_TOP )
		((BRadioButton *) FindView( "RB_top" ))->SetValue( TRUE );
	else if( ( resize & kResizeTBMask ) == B_FOLLOW_BOTTOM )
		((BRadioButton *) FindView( "RB_bottom" ))->SetValue( TRUE );
	else if( ( resize & kResizeTBMask ) == B_FOLLOW_TOP_BOTTOM )
		((BRadioButton *) FindView( "RB_topbottom" ))->SetValue( TRUE );
	else
		((BRadioButton *) FindView( "RB_tbcenter" ))->SetValue( TRUE );

	if( GetFrame()->GetTypeInfo()->type == EType_StringView )
	{
		// Set text
		BTextControl *pLabel = (BTextControl *) FindView( "TX_text" );
		if( pLabel )
		{
			char name[ kDITLStringTextLen ];
			strncpy( name, ((BStringView *) pControl)->Text(), kDITLStringTextLen );
			pLabel->SetText( name );
		}
		
		// set alignment
		alignment a = ((BStringView *) pControl)->Alignment();
		if( a == B_ALIGN_LEFT )
			((BRadioButton *) FindView( "RB_aleft" ))->SetValue( TRUE );
		else if( a == B_ALIGN_CENTER )
			((BRadioButton *) FindView( "RB_acenter" ))->SetValue( TRUE );
		else
			((BRadioButton *) FindView( "RB_aright" ))->SetValue( TRUE );
		
	}
	else
	{
		// Set label
		BTextControl *pLabel = (BTextControl *) FindView( "TX_label" );
		if( pLabel )
		{
			char name[ kDITLTextCLabelLen ];
			strncpy( name, ((BTextControl *) pControl)->Label(), kDITLTextCLabelLen );
			pLabel->SetText( name );
		}
	}
	
	Unlock();
}				


void CEditor::_Apply( void )
{
	BView *pControl = GetFrame()->GetControl();
	
	// update name
	BTextControl *pText = (BTextControl *) FindView( "TX_name" );
	if( pText )
	{
		char name[ kDITLViewNameLen ];
		strncpy( name, pText->Text(), kDITLViewNameLen );
		pControl->SetName( name );
	}
		
	// update resize
	BRadioButton *pRad;
	ulong resize = 0L;
	
	if( ((BRadioButton *) FindView( "RB_left" ))->Value() )
		resize = B_FOLLOW_LEFT;
	else if( ((BRadioButton *) FindView( "RB_right" ))->Value() )
		resize = B_FOLLOW_RIGHT;
	else if( ( pRad = (BRadioButton *) FindView( "RB_leftright" ) ) && pRad->Value() )
		resize = B_FOLLOW_LEFT_RIGHT;
	else
		resize = B_FOLLOW_H_CENTER;
	
	if( ((BRadioButton *) FindView( "RB_top" ))->Value() )
		resize |= B_FOLLOW_TOP;
	else if( ((BRadioButton *) FindView( "RB_bottom" ))->Value() )
		resize |= B_FOLLOW_BOTTOM;
	else if( ( pRad = (BRadioButton *) FindView( "RB_topbottom" ) ) && pRad->Value() )
		resize |= B_FOLLOW_TOP_BOTTOM;
	else
		resize |= B_FOLLOW_V_CENTER;

	GetFrame()->SetResizingMode( resize );
	pControl->SetResizingMode( resize );
	
	// Type specific info
	if( GetFrame()->GetTypeInfo()->type == EType_StringView )
	{
		BStringView *pView = (BStringView *) pControl;
		
		// Set text
		BTextControl *pLabel = (BTextControl *) FindView( "TX_text" );
		if( pLabel )
		{
			char name[ kDITLStringTextLen ];
			strncpy( name, pLabel->Text(), kDITLStringTextLen );
			pView->SetText( name );
		}
		
		// set alignment
		if( ((BRadioButton *) FindView( "RB_aleft" ))->Value() )
			pView->SetAlignment( B_ALIGN_LEFT );
		else if( ((BRadioButton *) FindView( "RB_acenter" ))->Value() )
			pView->SetAlignment( B_ALIGN_CENTER );
		else			
			pView->SetAlignment( B_ALIGN_RIGHT );
	}
	else
	{
		BTextControl *pText = (BTextControl *) pControl;
		
		// Set label
		BTextControl *pLabel = (BTextControl *) FindView( "TX_label" );
		if( pLabel )
		{
			char name[ kDITLTextCLabelLen ];
			strncpy( name, pLabel->Text(), kDITLTextCLabelLen );
			pText->SetLabel( name );
		}
		
		// set divider
		float d = pText->StringWidth( pText->Label() ) + 3;
		pText->SetDivider( d );
		
		// FIX BAD BE  Move text view over
		BPoint p = pText->Bounds().RightBottom();
		pText->ResizeTo( p.x, p.y );
//		BTextView *pTextView = (BTextView *) pText->ChildAt( 0 );
//		BPoint p = pTextView->Frame().LeftTop();
//		pTextView->MoveTo( d, p.y );
	}
}
