/*
	ViewEditorBase.cpp
	Copyright 1997 Nathan Dwyer, All Rights Reserved.
	
*/


#include "DITLWindow.h"
#include "ViewEditorBase.h"

//---------------------------- Constants

static const BRect default_rect( 100, 100, 200, 200 );

const EDItemType EType_View = 'DiVi';
const EDItemType EType_Box = 'DiBx';
const EDItemType EType_Button = 'DiBu';
const EDItemType EType_Checkbox = 'DiCh';
const EDItemType EType_RadioButton = 'DiRb';
const EDItemType EType_TextControl = 'DiTv';
const EDItemType EType_StringView = 'DiSv';
const EDItemType EType_ColorControl = 'DiCC';




//---------------------------- Prototypes


//---------------------------- Public methods

CEditorWindow::CEditorWindow( CDragFrame *pFrame, const char *dlgRsrc, ulong rsrcLen ) :
	BWindow( default_rect, "Edit", B_TITLED_WINDOW, 0L ),
	mpFrame( pFrame )
{
	printf( "::CEditorWindow\n" );
	
	_LoadInterface( dlgRsrc, rsrcLen );
}


bool CEditorWindow::QuitRequested()
{
	printf( "Quit Requested\n" );

	if( GetFrame() )
		GetFrame()->ClearEditWindow();
		
	return inherited::QuitRequested();
}


void CEditorWindow::MessageReceived( BMessage *pMsg )
{
	switch( pMsg->what )
	{
		case kApplyMsg:
			mpFrame->Window()->Lock();
			_Apply();
			mpFrame->Window()->Unlock();
			{
				BMessage msg( kUpdateRsrcMsg );
				msg.AddLong( "RsrcID", mpFrame->GetID() );
				
				mpFrame->Window()->MessageReceived( &msg );
			}
			break;
			
		default:
			inherited::MessageReceived( pMsg );
			break;
	}
}


struct SViewHolder
{
	public:
		SViewHolder( ulong _id, BView *_pView ) { id = _id; pView = _pView; }
		
		ulong id;
		BView *pView;
};
	

void CEditorWindow::_LoadInterface( const char *pData, ulong rsrcLen )
{
	if( pData )
	{
		const char *rsrcData = pData;
		BList viewList;
		
		// load window frame
		float w, h;
		w = *(float *) ( rsrcData + kDITLWindowFrameWOffset );
		h = *(float *) ( rsrcData + kDITLWindowFrameHOffset );
		ResizeTo( w, h );

		// load views
		rsrcData += kDITLHeaderSize;
		rsrcLen -= kDITLHeaderSize;
		
		while( rsrcLen )
		{
			long sz = *(long *) ( rsrcData + kDITLViewSize );
			ulong parentid = *(ulong *) ( rsrcData + kDITLViewParentID );
			ulong viewid = *(ulong *) ( rsrcData + kDITLViewID );

			BView *pView = _CreateView( rsrcData );
			
			// find parent in viewlist
			BView *pParent = NULL;
			for( long l=0; l<viewList.CountItems(); l++ )
			{
				SViewHolder *pHolder = (SViewHolder *) viewList.ItemAt( l );
				if( pHolder->id == parentid )
				{
					pParent = pHolder->pView;
					break;
				}
			}
			
			if( pParent )
				pParent->AddChild( pView );
			else
				AddChild( pView );

			SViewHolder *pHolder = new SViewHolder( viewid, pView );	
			viewList.AddItem( (void *) pHolder );
					
			rsrcData += sz;
			rsrcLen -= sz;
		}

		// Get rid of allocated data
		while( viewList.CountItems() )
		{
			SViewHolder *pHolder = (SViewHolder *) viewList.RemoveItem( 0L );
			delete pHolder;
		}
		
		// HACK -- add apply message to BN_apply
		BButton *pApply = (BButton *) FindView( "BN_apply" );
		if( pApply )
			pApply->SetMessage( new BMessage( kApplyMsg ) );
	}
}



BView *CEditorWindow::_CreateView( const char *pData )
{
	BView *pRet = NULL;
	
	ulong type = *(ulong *) ( pData + kDITLViewType );
	BRect frame;
	frame.left = *(float *) ( pData + kDITLViewFrameL );
	frame.top = *(float *) ( pData + kDITLViewFrameT );
	frame.right = *(float *) ( pData + kDITLViewFrameR );
	frame.bottom = *(float *) ( pData + kDITLViewFrameB );
	const char *pName = pData + kDITLViewName;
	long resize = *(long *) ( pData + kDITLViewResize );

	switch( type )
	{
		case EType_None:
			pRet = NULL;
			break;
			
		case EType_View:
			pRet = new BView( frame, pName, resize, B_WILL_DRAW );
			pRet->SetViewColor(	*(rgb_color *) ( pData + kDITLViewColor ) );
			break;
			
		case EType_Box:
			pRet = new BBox( frame, pName );
			pRet->SetViewColor(	*(rgb_color *) ( pData + kDITLViewColor ) );
			((BBox *) pRet)->SetLabel( pData + kDITLBoxTitle );	
			break;
			
		case EType_RadioButton:
			pRet = new BRadioButton( frame, pName, pData + kDITLControlTitle, NULL );
			break;
			
		case EType_Checkbox:
			pRet = new BCheckBox( frame, pName, pData + kDITLControlTitle, NULL );
			break;
			
		case EType_Button:
			pRet = new BButton( frame, pName, pData + kDITLControlTitle, NULL );
			((BButton *) pRet)->MakeDefault( (bool) *(long *) ( pData + kDITLButtonDefault ) );
			break;
			
		case EType_StringView:
			pRet = new BStringView( frame, pName, pData + kDITLStringText );
			((BStringView *) pRet)->SetAlignment( *(alignment *) ( pData + kDITLStringAlign ) );
			break;
			
		case EType_TextControl:
			pRet = new BTextControl( frame, pName, pData + kDITLTextCLabel, "text", NULL  );
			((BTextControl *) pRet)->SetDivider( *(float *) ( pData + kDITLTextCDivider ) );
			break;
			
		case EType_ColorControl:
			pRet = new BColorControl( frame.LeftTop(), B_CELLS_32x8, 6, pName );
			break;
			
		default:
			printf( "Bad control type\n" );
			break;
	}

	if( pRet )
		pRet->SetResizingMode( resize );

	return pRet;
}	


