/*
	ViewPlugin.cpp
	Copyright 1997 Nathan Dwyer, All Rights Reserved.
*/


#include "PluginAPI.h"
#include "ViewEditorBase.h"


const EDItemType EType_View = 'DiVi';
const EDItemType EType_Box = 'DiBx';
const EDItemType EType_ColorControl = 'DiCC';
	
const ulong EType_NumTypes = 3;



static SItemInfo ControlInfo[ EType_NumTypes ] =
{
	{ EType_View, "view", { 'V', '\0' }, 10, 10, 0, 0, 0, 0, 0, 0, 90, 60, TRUE, 104, { 0, 0 }, 0 },
	{ EType_Box, "box", { 'X', '\0' }, 10, 10, 0, 0, -2, -2, 2, 2, 90, 60, TRUE, 168, { 0, 0 }, 0 },
	{ EType_ColorControl, "color", { 'P', '\0' }, 194, 50, 194, 50, -1, -1, 1, 1, 194, 50, FALSE, 164, { 0, 0 }, 0 }
};



class CEditor : public CEditorWindow
{
	public:
		CEditor( CDragFrame *pFrame, const char *rsrcData, ulong rsrcLen );

		virtual void Init( void );
		
	protected:
		virtual void _Apply( void );
};



#pragma export on


SItemInfo *GetViewInfo( long *pCnt )
{
	*pCnt = EType_NumTypes;
	return ControlInfo;
}




BView *MakeView( long type, BRect frame )
{
	BView *pControl = NULL;
	
	switch( type )
	{
		case EType_View:
			pControl = new BView( frame, NULL, B_FOLLOW_TOP | B_FOLLOW_LEFT, B_WILL_DRAW );
			break;
			
		case EType_Box:
			pControl = new BBox( frame, NULL );
			((BBox *) pControl)->SetLabel( "Box:" );			
			break;
			
		case EType_ColorControl:
			pControl = new BColorControl( frame.LeftTop(), B_CELLS_32x8, 6, NULL );
			pControl->SetResizingMode( B_FOLLOW_LEFT | B_FOLLOW_TOP );
			break;
	}
	
	return pControl;

}


void ReadView( CDragFrame *pFrame, DBlock& buffer, long offset )
{
	if( pFrame->GetTypeInfo()->type != EType_ColorControl )
	{
		long l;
		BView *pView = pFrame->GetControl();
	
		// read color
		rgb_color color = *(rgb_color *) buffer( offset + kDITLViewColor, sizeof(long) ).Data( l );
		pView->SetViewColor( color );
		
		// Read label for BBox
		if( pFrame->GetTypeInfo()->type == EType_Box )
		{
			BBox *pBox = (BBox *) pView;
	
			char name[ kDITLBoxTitleLen ];
			strncpy( name, buffer( offset + kDITLBoxTitle, kDITLBoxTitleLen ).Data( l ), sizeof(name) );
			pBox->SetLabel( name );
		}
	}
}


void WriteView( CDragFrame *pFrame, DBlock& buffer, long offset )
{
	if( pFrame->GetTypeInfo()->type != EType_ColorControl )
	{
		BView *pView = pFrame->GetControl();
		
		// write color
		rgb_color color = pView->ViewColor();
		buffer( offset + kDITLViewColor, sizeof(long) ) = DBlock( (char *) &color, sizeof(long) );
		
		// write out label for BBox
		if( pFrame->GetTypeInfo()->type == EType_Box )
		{
			BBox *pBox = (BBox *) pView;
			
			char buff[ kDITLBoxTitleLen ];
			strcpy( buff, pBox->Label() );
			buffer( offset + kDITLBoxTitle, kDITLBoxTitleLen ) = DBlock( buff, kDITLBoxTitleLen );
		}
	}
}


BWindow *EditView( CDragFrame *pFrame )
{
	char *rsrcData = NULL;
	long rsrcLen = 0L;
	
	// find record ref for this frame type
	const SItemInfo *pInfo = pFrame->GetTypeInfo();
	
	// get resource
	BResourceFile rsrcFile( pInfo->pluginRef );
	if( B_NO_ERROR == rsrcFile.Open( B_READ_ONLY ) )
	{
		long dlgID = 2;
		
		if( pInfo->type == EType_View )
			dlgID = 0;
		else if( pInfo->type == EType_Box )
			dlgID = 1;
			
		rsrcData = (char *) rsrcFile.FindResource( 'DITL', dlgID, &rsrcLen );
		rsrcFile.Close();
	}
	
	CEditorWindow *pWin = NULL;
	
	if( rsrcData )
	{
		// create and show editor window
		pWin = new CEditor( pFrame, rsrcData, rsrcLen );	
		pWin->Init();
		
		// clean up
		free( rsrcData );
	}		
	
	return pWin;
}

#pragma export reset


//-------------------------------- Editor Class

CEditor::CEditor( CDragFrame *pFrame, const char *rsrcData, ulong rsrcLen ) :
	CEditorWindow( pFrame, rsrcData, rsrcLen )
{
}
	


void CEditor::Init( void )
{
	Lock();
	
	BView *pControl = GetFrame()->GetControl();
	
	// set name
	BTextControl *pText = (BTextControl *) FindView( "TX_name" );
	if( pText )
	{
		char name[ kDITLViewNameLen ];
		strncpy( name, pControl->Name(), kDITLViewNameLen );
		pText->SetText( name );
	}
		
	// set resize
	ulong resize = pControl->ResizingMode();
	
	const ulong kResizeLRMask = 0x00000f0f;
	if( ( resize & kResizeLRMask ) == B_FOLLOW_LEFT )
		((BRadioButton *) FindView( "RB_left" ))->SetValue( TRUE );
	else if( ( resize & kResizeLRMask ) == B_FOLLOW_RIGHT )
		((BRadioButton *) FindView( "RB_right" ))->SetValue( TRUE );
	else if( ( resize & kResizeLRMask ) == B_FOLLOW_LEFT_RIGHT )
	{
		BRadioButton *pRad = (BRadioButton *) FindView( "RB_leftright" );
		if( pRad )
			pRad->SetValue( TRUE );
	}
	else
		((BRadioButton *) FindView( "RB_lrcenter" ))->SetValue( TRUE );
		
	const ulong kResizeTBMask = 0x0000f0f0;
	if( ( resize & kResizeTBMask ) == B_FOLLOW_TOP )
		((BRadioButton *) FindView( "RB_top" ))->SetValue( TRUE );
	else if( ( resize & kResizeTBMask ) == B_FOLLOW_BOTTOM )
		((BRadioButton *) FindView( "RB_bottom" ))->SetValue( TRUE );
	else if( ( resize & kResizeTBMask ) == B_FOLLOW_TOP_BOTTOM )
	{
		BRadioButton *pRad = (BRadioButton *) FindView( "RB_topbottom" );
		if( pRad )
			pRad->SetValue( TRUE );
	}
	else
		((BRadioButton *) FindView( "RB_tbcenter" ))->SetValue( TRUE );

	// Type specific data
	const SItemInfo *pInfo = GetFrame()->GetTypeInfo();
	if( pInfo->type != EType_ColorControl )
	{
		// set ViewColor
		BColorControl *pColor = (BColorControl *) FindView( "PL_color" );
		if( pColor )
			pColor->SetValue( pControl->ViewColor() );
		
		if( pInfo->type == EType_Box )
		{
			// Set label
			BTextControl *pLabel = (BTextControl *) FindView( "TX_label" );
			
			printf( "label: %p\n", pLabel );
			if( pLabel )
			{
				char name[ kDITLBoxTitleLen ];
				strncpy( name, ((BBox *) pControl)->Label(), kDITLBoxTitleLen );
				pLabel->SetText( name );
			}
			
		}
	}
		
	Unlock();
}				


void CEditor::_Apply( void )
{
	BView *pControl = GetFrame()->GetControl();
	
	// update name
	BTextControl *pText = (BTextControl *) FindView( "TX_name" );
	if( pText )
	{
		char name[ kDITLViewNameLen ];
		strncpy( name, pText->Text(), kDITLViewNameLen );
		pControl->SetName( name );
	}
		
	// update resize
	BRadioButton *pRad;
	ulong resize = 0L;
	
	if( ((BRadioButton *) FindView( "RB_left" ))->Value() )
		resize = B_FOLLOW_LEFT;
	else if( ((BRadioButton *) FindView( "RB_right" ))->Value() )
		resize = B_FOLLOW_RIGHT;
	else if( ( pRad = (BRadioButton *) FindView( "RB_leftright" ) ) && pRad->Value() )
		resize = B_FOLLOW_LEFT_RIGHT;
	else
		resize = B_FOLLOW_H_CENTER;
	
	if( ((BRadioButton *) FindView( "RB_top" ))->Value() )
		resize |= B_FOLLOW_TOP;
	else if( ((BRadioButton *) FindView( "RB_bottom" ))->Value() )
		resize |= B_FOLLOW_BOTTOM;
	else if( ( pRad = (BRadioButton *) FindView( "RB_topbottom" ) ) && pRad->Value() )
		resize |= B_FOLLOW_TOP_BOTTOM;
	else
		resize |= B_FOLLOW_V_CENTER;

	GetFrame()->SetResizingMode( resize );
	pControl->SetResizingMode( resize );
	
	// Type specific data	
	const SItemInfo *pInfo = GetFrame()->GetTypeInfo();
	if( pInfo->type != EType_ColorControl )
	{
		// Update ViewColor
		BColorControl *pColor = (BColorControl *) FindView( "PL_color" );
		if( pColor )
			pControl->SetViewColor( pColor->ValueAsColor() );
	
		if( pInfo->type == EType_Box )
		{
			// Set label
			BTextControl *pLabel = (BTextControl *) FindView( "TX_label" );
			
			if( pLabel )
			{
				char name[ kDITLBoxTitleLen ];
				strncpy( name, pLabel->Text(), kDITLBoxTitleLen );
				((BBox *) pControl)->SetLabel( name );
			}
		}		
	}
}
